/*
 * libsapilektor.h - libsapilektor headers
 * Copyright (C) Bohdan R. Rau 2012 <ethanak@polip.com>
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */
#ifndef ETH_LIBSAPILEKTOR_H
#define ETH_LIBSAPILEKTOR_H 1

struct SAPI_data {
    int sex;    // SAPI_MALE or SAPI_FEMALE 
    int freq;   // may be 0 (uninitialized) before first synthesized sentence
    double silmpx; // for silence detector
    char lang[8];    // language code (ISO or Microsoft)
};


#define SAPI_MALE 0
#define SAPI_FEMALE 1
#define SAPI_CHILD 2

#ifdef __cplusplus
extern "C" {
#endif

/*
 * In all functions the "error" parameter must be null or pointer to
 * char * variable. In case of error, newly allocated string will be
 * assigned to this variable. Application must free this string
 * when not needed.
 */

/*
 * Open voice and connect to server.
 * name must be one of section names (case insensitive)
 */

struct SAPI_data *SAPI_open(
    char *name,
    char **error);


/*
 * Open voice and connect to server.
 * name must be one of section names (case insensitive)
 * host (if not NULL) overrides host parameter from config
 * port (if not zero) overrides port parameter from config
 */

struct SAPI_data *SAPI_openNetwork(
    char *name,
    char *host,
    int port,
    char **error);

/*
 * Close connection and free handle structure
 */

void SAPI_close(
    struct SAPI_data *handle);

short *SAPI_getWave(
    struct SAPI_data *handle,
    char *text,
    int *length,
    char **error);

int SAPI_trimWave(
    struct SAPI_data *handle,
    short *wave,
    int length,
    int *offset, // count of silent samples at start
    int *rest);  // count of silent samples at end


    
short *SAPI_getSentence(struct SAPI_data *handle,
    char *sentence,
    int *sample_offset, // count of silent samples at start
    int *sample_count,  // count of usable samples
    int *sample_rest,   // count of silent samples at end
    double pause_factor, // factor for pause shortener
    char **error);

int SAPI_trimLastPart(struct SAPI_data *handle,
    short *wave,
    int length,
    int *out_length,
    int *rest);

short *SAPI_Repauser(struct SAPI_data *handle,
    short *wave,
    int *len,
    double factor,
    int *offset,
    int *rest);

char ** SAPI_voices(void);
char **SAPI_voicesFull(void);

void SAPI_enableSSML(struct SAPI_data *handle,int enable);
void SAPI_debugFile(struct SAPI_data *handle,FILE *file);

const char * const SAPI_version(void);

#ifdef __cplusplus
}
#endif

#endif
