/*
 * sapi_s4l.c - sapi4linux driver for libsapilektor
 * Copyright (C) Bohdan R. Rau 2012 <ethanak@polip.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */


static struct SAPI_data *sapiOpenS4l(struct sapiConf *conf,
	struct sapiSection *ise,
	char *param_host,
	int param_port,
	char **error)
{
    char *pm;
    int port;
    FILE *f;
    char *voice;
    struct SAPI_PrivData *ipd;
    char linebuf[1024];
    char *lpos;
    int vid,fid,gender=0;

    voice=sapi_ConfParam(ise,"voice");
    if (!voice) {
        if (error) {
            *error=malloc(strlen(ise->name)+128);
            sprintf(*error,"No 'voice' parameter in section [%s]",ise->name);
        }
        sapi_FreeConf(conf);
        return NULL;
    }

    ipd=malloc(sizeof(*ipd));
    memset(ipd,0,sizeof(*ipd));
    ipd->conf=conf;
    ipd->sect=ise;
    ipd->mode=SAPI_MODE_S4L;
    pm=sapi_ConfParam(ise,"silmpx");

    if (pm) ipd->public.silmpx=strtod(pm,NULL);
    else ipd->public.silmpx=1.0;

    pm=sapi_ConfParam(ise,"timeout");
    if (!pm) ipd->timeout=1000;
    else {
		ipd->timeout=strtol(pm,NULL,10);
		if (ipd->timeout < 200) ipd->timeout=200;
		else if (ipd->timeout > 5000) ipd->timeout=5000;
	}

    if (param_port) {
        ipd->rockbox_port=param_port;
    }
    else {
        pm=sapi_ConfParam(ise,"port");
        if (!pm) port=8765;
        else port=strtol(pm,NULL,10);
        ipd->rockbox_port=port;
    }
    
    pm=sapi_ConfParam(ise,"gender");
    if (pm) {
	ipd ->public.sex &= ~1;
	if (tolower(*pm)=='f') ipd->public.sex |=1;
    }
    pm=sapi_ConfParam(ise,"age");
    if (pm) {
	ipd ->public.sex &= ~2;
	if (tolower(*pm)=='f') ipd->public.sex |=2;
    }
    pm=sapi_ConfParam(ise,"lang");
    if (pm && strlen(pm) < 8) {
	strcpy(ipd->public.lang,pm);
    }

    if (param_host) pm=param_host;
    else pm=sapi_ConfParam(ise,"host");
    if (!pm) pm="127.0.0.1";
#if 0    
    if (strcmp(pm,"127.0.0.1") && strcmp(pm,"localhost")) {
        ipd->rockbox_autospawn=aspawn=0;
    }
#endif
    if (!sapi_InitAddr(pm,ipd->rockbox_port,&ipd->rockbox_sinadr)) {
        if(error) {
            *error=malloc(strlen(ise->name)+strlen(pm)+128);
            sprintf(*error,"Bad address %s:%d in section [%s]",
                pm,port,ise->name);
        }
        free(ipd);
        sapi_FreeConf(conf);
        return NULL;
    }
/*
    f=sapi_ConnectFile(&ipd->rockbox_sinadr,ipd->timeout);
    if (!f) {
        if (error) *error=strdup(strerror(errno));
        free(ipd);
        sapi_FreeConf(conf);
        return NULL;
    }
*/
    ipd->s4l_voice=strdup(voice);
    return (struct SAPI_data *)ipd;
}

static int sapi_s4l_cmd(FILE *f,char *cmd,char *buf)
{
    char *c;int v;
    //printf("Command %s\n",cmd);
    fprintf(f,"%s\n",cmd);
    fflush(f);
    if (!fgets(buf,256,f)) return -1;
    v=strtol(buf,&c,10);
    while (*c && isspace(*c)) c++;
    if (c != buf) strcpy(buf,c);
    //printf("Response %d %s\n",v,buf);
    return v;
}

static void sapi_S4lClose(struct SAPI_PrivData *handle)
{
	xfree(handle->s4l_voice);
}

static void sapi_S4lFreeWHD(FILE *f)
{
	fclose(f);
}

static FILE *sapi_S4lInitWave(struct SAPI_PrivData *handle,char *str,char **error)
{
    int rc;
    FILE *f;
    char *ustr,buf[1024];
    str=trim(str);
    while (*str && isspace(*str)) str++;
    if (!*str) {
        if (error) *error=strdup("Empty sentences");
        return NULL;
    }
    f=sapi_ConnectFile(&handle->rockbox_sinadr,handle->timeout);
    if (!f) {
        if (error) *error=strdup(strerror(errno));
        return NULL;
    }
    sprintf(buf,"setvoice %s",handle->s4l_voice);
    rc=sapi_rockbox_cmd(f,buf,buf);
    if (rc<0) {
        if (error) *error=strdup(strerror(errno));
        fclose(f);return NULL;
    }
    fprintf(f,"speak\n");
    while (str && *str) {
		ustr=strpbrk(str,"\r\n");
		if (!ustr) break;
		if (ustr == str+1 && *str=='.') {
			fprintf(f,".%-*.*s\n",(int)(ustr-str),(int)(ustr-str),str);
		}
		else {
			fprintf(f,"%-*.*s\n",(int)(ustr-str),(int)(ustr-str),str);
		}
		if (*ustr == '\r') ustr++;
		if (*ustr == '\n') ustr++;
		str=ustr;
	}
	if (str) fprintf(f,"%s\n",str);
    fprintf(f,".\n");
    fflush(f);
    return f;
}

static short *sapi_S4lFetchWave(struct SAPI_PrivData *handle,FILE *f,int *sample_length,char **error)
{
    char buf[4096],*bptr;
    char *wavestring;
    int stringlen,rc;
    
    void bs2n(char *c)
    {
		char *d;
		for (d=c;*c;) {
			if (*c != '\\' || c[1] !='n') {
				*d++=*c++;
				continue;
			}
			*d++='\n';
			c+=2;
		}
		*d=0;
	}

    if (!fgets(buf,4096,f)) {
        if (error) *error=strdup(strerror(errno));
        fclose(f);return NULL;
    }
    rc=strtol(buf,&bptr,10);
    if (rc != 200 || *bptr++!=':') {
        fclose(f);
        if (error) {
			*error=strdup(trim(buf));
			bs2n(*error);
		}
        return NULL;
    }
    bptr=strchr(bptr,':');
    if (!bptr) {
        fclose(f);
        if (error) *error=strdup(trim(buf));
        return NULL;
    }
    bptr++;
    stringlen=strtol(bptr,NULL,10);
read_wave:
    wavestring=malloc(stringlen);
    rc=fread(wavestring,1,stringlen,f);
    fclose(f);
    rc=*(long *)(wavestring+16);
    handle->public.freq=*(long *)(wavestring+24);
    rc+=28;
    stringlen-=rc;
    memcpy(wavestring,wavestring+rc,stringlen);
    *sample_length=stringlen/2;
    return (short *) wavestring;
}

static short *sapi_S4lGetWave(struct SAPI_PrivData *handle,char *str,int *sample_length,char **error)
{
    FILE *f;
    f=sapi_S4lInitWave(handle,str,error);
    if (!f) return NULL;
    return sapi_S4lFetchWave(handle,f,sample_length,error);
}
