/*
 * sapitest - libsapilektor tester
 * Copyright (C) Bohdan R. Rau 2012 <ethanak@polip.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libsapilektor.h>
#include <getopt.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/stat.h>
#include "sonic.h"
#include "audio.h"

double pause_factor = 1.0;
double speed = 1.0;
char *voice=NULL;
char **voices;

void helpme(char *program, int estatus)
{
    int i;
    fprintf(stderr,"SAPI lektor v %s\n",SAPI_version());
    fprintf(stderr,"Usage: %s [options] [-f <name> | text to say]\n",program);
    fprintf(stderr,"Options are:\n\
  -p <float> - pause factor (0.5 .. 1.0)\n\
  -s <float> - speed (0.5 .. 3.0)\n\
  -v <name>  - voice name (");
    for (i=0;voices[i];i++) fprintf(stderr,"%s%s",i?", ":"",voices[i]);

   fprintf(stderr,")\n\
  -l         - enable SSML in command line (excludes -f)\n\
  -f <name>  - SSML input file (excludes -l)\n\
  -h         - this help\n");
   exit(estatus);
}

struct SAPI_data *vox;
static sonicStream sonic;

void play_nice(short *wave,int nsamples)
{
    if (sonic) {
        short buf[32768];
        int n,inlen;
        while (nsamples > 0) {
            inlen=16384;
            if (inlen > nsamples) inlen=nsamples;
            sonicWriteShortToStream(sonic,wave,inlen);
            wave+=inlen;
            nsamples-=inlen;
            for (;;) {
                n=sonicReadShortFromStream(sonic,buf,32768);
                if (n<=0) break;
                play_audio(buf,n);
            }
        }
        return;
    }
    play_audio(wave,nsamples);
}

int main(int argc, char *argv[])
{
    static char *optstr="hv:s:p:lf:";
    int opt,idx,i;
    char *err;
    short *wave=NULL;
    int offset;
    int count;
    int rest;
    char *str;
    int ssml=0;
    char *infname=NULL;
    
    voices=SAPI_voices();
    if (!voices || !*voices) {
        fprintf(stderr,"No SAPI voices declared\n");
        exit(1);
    }
    double darg(double minval, double maxval) {
        char *d;
        double val=strtod(optarg,&d);
        if (d == optarg || val < minval || val > maxval) helpme(argv[0],1);
        return val;
    }
    while ((opt = getopt(argc,argv,optstr)) != -1) {
        switch(opt) {
            case 'h': helpme(argv[0],0);break;
            case 'v': voice=strdup(optarg);break;
            case 's': speed=darg(0.5,3.0);break;
            case 'p': pause_factor=darg(0.2,1.0);break;
            case 'l': if (infname) helpme(argv[0],1);ssml=1;break;
            case 'f': if (ssml) helpme(argv[0],1);infname=strdup(optarg);break;
            default: helpme(argv[0],1);
        }
    }
    if (!voice) voice=voices[0];
    else {
        int i;
        for (i=0;voices[i];i++) if (!strcasecmp(voices[i],voice)) break;
        if (!voices[i]) helpme(argv[0],0);
    }
    idx=optind;
    
    if (idx >= argc) {
		if (!infname) str=strdup("Test");
		else if (!strcmp(infname,"-")) {
			int bsize=1024;
			int rsize=0;
			str=malloc(bsize);
			for (;;) {
				int d=read(0,str+rsize,bsize-rsize);
				if (d <= 0) break;
				rsize+=d;
				if (rsize == bsize) {
					bsize = 2*bsize;
					str=realloc(str,bsize);
				}
			}
			str[rsize]=0;
		}
		else {
			struct stat sb;
			int fd;
			if (stat(infname,&sb)) {
				perror(infname);
				exit(1);
			}
			fd=open(infname,O_RDONLY);
			if (fd < 0) {
				perror(infname);
				exit(1);
			}
			str=malloc(sb.st_size+1);
			if (read(fd,str,sb.st_size) != sb.st_size) {
				perror(infname);
				exit(1);
			}
			close(fd);
			str[sb.st_size]=0;
		}
    }
    else {
        int len=1024;
        if (infname) helpme(argv[0],1);
        for (i=idx;i<argc;i++) len += strlen(argv[1])+1;
        str=malloc(len);
        for (i=idx;i<argc;i++) {
            if (i>idx) strcat(str," ");
            else *str=0;
            strcat(str,argv[i]);
        }
    }
    vox=SAPI_open(voice,&err);
    if (!vox) {
        fprintf(stderr,"Error: %s\n",err);
        exit(1);
    }
    SAPI_enableSSML(vox,ssml);
    wave=SAPI_getSentence(vox,str,&offset,&count,&rest,pause_factor,&err);
    if (!wave) {
        fprintf(stderr,"Error: %s\n",err);
        exit(1);
    }
    if (speed != 1.0) {
        sonic=sonicCreateStream(vox->freq,1);
        sonicSetSpeed(sonic,speed);
    }
    init_ao(vox->freq);
    play_nice(wave,offset+count+rest);
    audio_close_device();
    exit(0);
}
